//
//  MarvelCharacters.swift
//  TabBarUI
//
//  Created by Marcus Macbook Pro on 8/7/21.
//

import SwiftUI
import SDWebImageSwiftUI

struct MarvelCharactersView: View{
    @EnvironmentObject var homeData: ContentViewModel
    var body: some View {
        //Nationgation View..
        
        NavigationView{
           
            ScrollView(.vertical, showsIndicators: true, content:{
                VStack(spacing: 15){
                    
                    //Search Bar...
                    HStack(spacing: 10) {
                       
                        Image( systemName: "magnifyingglass")
                            .foregroundColor(.gray)
                        
                        TextField("Search Character", text: $homeData.searchQuery)
                            .autocapitalization(.none)
                            .disableAutocorrection(true)
                    }
                    .padding(.vertical,10)
                    .padding(.horizontal)
                    .background(Color.white)
                    
                    //Shadows...
                    .shadow(color: Color.black.opacity(0.06), radius: 5, x: 5, y:5)
                    .shadow(color: Color.black.opacity(0.06), radius: 5, x: -5, y: -5)
                }
                .padding()
                
                if let characters = homeData.fetchedPorn{
                    
                    if characters.isEmpty{
                        
                        //No Results
                        Text("No Result Found")
                            .padding(.top,20)
                    }
                    else{
                        
                        // Displaying results
                        ForEach(characters){data in
                            
                           CharacterRowView(character: data)
                            
                            
                        }
                    }
                }
                else{
                    if homeData.searchQuery != ""{
                    //Lokading Screem
                    ProgressView()
                        .padding(.top,20)
                    }
                }
            })
            .navigationTitle("Marvel Characters")
        }
    }
}

struct MarvelCharactersView_Previews: PreviewProvider {
    static var previews: some View {
        ContentView()
    }
}

struct CharacterRowView: View{
    
    var character: VideoVideo
    var body: some View{
        
        HStack(alignment: .top,spacing: 15){
            
            WebImage(url: extractImage(data: character.thumb))
                .resizable()
                .aspectRatio(contentMode: .fill)
                .frame(width: 150, height: 150)
                .cornerRadius(8)
            
            VStack(alignment: .leading, spacing: 8, content: {
                
                Text(character.title)
                    .font(.title3)
                    .fontWeight(.bold)
                
                Text(character.rating)
                    .font(.title3)
                    .fontWeight(.bold)
                    .lineLimit(4)
                    .multilineTextAlignment(.leading)
                
                
               //Links
                HStack(spacing: 10){
                    ForEach(character.url,id: \.self){
                        data in
                        NavigationLink(
                            destination: WebView(url: extractURL(data:data))
                                .navigationTitle(extractURLType(data:data)),
                            label:{
                                Text(extractURLType(data: data))
                            })
                        
                    }
                    
                }
                

                
                
            })
            
            Spacer(minLength: 0)
        }
        
        .padding(.horizontal)
    }
    
    func extractImage(data:[String: String])-> URL{
        
        // combing both and forming image
        
        let path = data["path"] ?? ""
        let ext = data["extension"] ?? ""
        
        return URL(string: "\(path).\(ext)")!
        
    }
    
    func extractURL(data: [String:String])->URL{
     let url = data["url"] ?? ""
        
        return URL(string: url)!
        
    }
    
    func extractURLType(data: [String:String])->String{
        
        let type = data["type"] ?? ""
        
        return type.capitalized
    }
}

